from java.util import Date
from java.lang import System

def fdLog(methodName, message):
    print str(Date()) + " - " + str(methodName) + " - " + str(message)

# Using java.lang.System as os.getenv has issues at times
# in WLST script
def fdGetEnv(environmentKey):
    val=System.getenv(environmentKey)
    if val == None:
        raise "Environment variable not found " + str(environmentKey)
    return val

def fdGetEnvOfDefault(environmentKey, defaultValue):
    val=System.getenv(environmentKey)
    if val == None:
        val=defaultValue
    return val
    
def fdDisconnect():
    thisMethodName="fdDisconnect"
    fdLog(thisMethodName, "Entering")
    disconnect()
    fdLog(thisMethodName, "Exiting")

def fdConnect():
    thisMethodName="fdConnect"
    fdLog(thisMethodName, "Entering")
    
    user=fdGetEnv('FDWLS_ADMIN_USER')
    password=fdGetEnv('FDWLS_ADMIN_PASSWORD')
    listenaddress=fdGetEnv('FDWLS_ADMIN_LISTEN_ADDRESS')
    listenport=fdGetEnv('FDWLS_ADMIN_LISTEN_PORT')
    isusessl=fdGetEnvOfDefault('FDWLS_ADMIN_USE_SSL', False)
    adminurl="t3://" + str(listenaddress) + ":" + str(listenport)
    if isusessl == True:
        adminurl="t3s://" + str(listenaddress) + ":" + str(listenport)

    connect(user,password,adminurl)
    
    fdLog(thisMethodName, "Exiting")

def fdReadDomain():
    thisMethodName="fdReadDomain"
    
    domainDir=fdGetEnv('FDWLS_ADMIN_DOMAIN_DIRECTORY')
    fdLog(thisMethodName, "Reading domain from " + str(domainDir))
    if os.path.exists(domainDir) == False:
        raise "Domain directory does not exists " + str(domainDir)
    
    readDomain(domainDir)
    
def fdWriteDomain():
    thisMethodName="fdWriteDomain"
    
    domainDir=fdGetEnv('FDWLS_ADMIN_DOMAIN_DIRECTORY')
    fdLog(thisMethodName, "Writing domain to " + str(domainDir))
    
    writeDomain(domainDir)

def fdUpdateDomain():
    thisMethodName="fdWriteDomain"
    
    domainDir=fdGetEnv('FDWLS_ADMIN_DOMAIN_DIRECTORY')
    fdLog(thisMethodName, "Updating domain " + str(domainDir))
    
    updateDomain()
    closeDomain()

def createBootPropertiesFile(directoryPath,fileName, username, password):
    # writing unencryted credentials, it will be encrypted when server is started.
    thisMethodName="createBootPropertiesFile"
    fdLog(thisMethodName, "Create boot.properties file in  " + str(directoryPath))
    
    serverDir = File(directoryPath)
    bool = serverDir.mkdirs()
    fileNew=open(directoryPath + '/'+fileName, 'w')
    fileNew.write('username=%s\n' % username)
    fileNew.write('password=%s\n' % password)
    fileNew.flush()
    fileNew.close()

# FIXME - implement derive logic to support 11g
def deriveWebLogicTemplateFilePath():
    thisMethodName="deriveWebLogicTemplateFilePath"
    
    oracleMWHome=fdGetEnv('FD_ORACLE_MW_HOME')
    templateFilePath=oracleMWHome + "/wlserver/common/templates/wls/wls.jar"
    
    fdLog(thisMethodName, "WebLogic template file path " + str(templateFilePath))
    if os.path.exists(templateFilePath) == False:
        raise "WebLogic template not found " + str(templateFilePath)

    return templateFilePath

# FIXME - implement derive logic to support 11g and other version of 12c
def deriveADFTemplateFilePath():
    thisMethodName="deriveADFTemplateFilePath"
    
    oracleMWHome=fdGetEnv('FD_ORACLE_MW_HOME')
    templateFilePath=oracleMWHome + "/oracle_common/common/templates/wls/oracle.jrf_template_12.1.3.jar"
    
    fdLog(thisMethodName, "ADF template file path " + str(templateFilePath))
    if os.path.exists(templateFilePath) == False:
        raise "ADF template not found " + str(templateFilePath)

    return templateFilePath

# FIXME - implement derive logic to support 11g and other version of 12c
def deriveFMWEMTemplateFilePath():
    thisMethodName="deriveFMWEMTemplateFilePath"
    
    oracleMWHome=fdGetEnv('FD_ORACLE_MW_HOME')
    templateFilePath=oracleMWHome + "/em/common/templates/wls/oracle.em_wls_template_12.1.3.jar"
    
    fdLog(thisMethodName, "FMW EM template file path " + str(templateFilePath))
    if os.path.exists(templateFilePath) == False:
        raise "FMW EM template not found " + str(templateFilePath)

    return templateFilePath

# Common code - ends here....

thisMethodName="main"
# update these variables if necessary
DEVELOPMENT_MODE=False

# first get all values that we may need ( using Property names defined by FlexDeploy )
domainName=fdGetEnv('FDWLS_DOMAIN_NAME')
fdLog(thisMethodName, "domainName=" + str(domainName))

domainDir=fdGetEnv('FDWLS_ADMIN_DOMAIN_DIRECTORY')
fdLog(thisMethodName, "domainDir=" + str(domainDir))

adminServerUser=fdGetEnv('FDWLS_ADMIN_USER')
fdLog(thisMethodName, "adminServerUser=" + str(adminServerUser))

adminServerPassword=fdGetEnv('FDWLS_ADMIN_PASSWORD')

adminServerListenaddress=fdGetEnv('FDWLS_ADMIN_LISTEN_ADDRESS')
fdLog(thisMethodName, "adminServerListenaddress=" + str(adminServerListenaddress))

adminServerListenport=fdGetEnv('FDWLS_ADMIN_LISTEN_PORT')
fdLog(thisMethodName, "adminServerListenport=" + str(adminServerListenport))

oracleMWHome=fdGetEnv('FD_ORACLE_MW_HOME')
fdLog(thisMethodName, "oracleMWHome=" + str(oracleMWHome))

applicationsHome=fdGetEnv('FDWLS_BASE_INSTALL_DIRECTORY')
fdLog(thisMethodName, "applicationsHome=" + str(applicationsHome))

nmHost=fdGetEnv('FDWLS_NODEMANAGER_HOST')
fdLog(thisMethodName, "nmHost=" + str(nmHost))

nmName=nmHost.split('.')[0]
fdLog(thisMethodName, "nmName=" + str(nmName))

nmPort=fdGetEnv('FDWLS_NODEMANAGER_PORT')
fdLog(thisMethodName, "nmPort=" + str(nmPort))

nmUser=fdGetEnv('FDWLS_NODEMANAGER_USER')
fdLog(thisMethodName, "nmUser=" + str(nmUser))

nmPassword=fdGetEnv('FDWLS_NODEMANAGER_PASSWORD')

nmType=fdGetEnv('FDWLS_NODEMANAGER_TYPE')
fdLog(thisMethodName, "nmType=" + str(nmType))

if domainDir.endswith(domainName) == False:
    raise "invalid domain dir, domain name must be last folder " + str(domainDir)

if applicationsHome.endswith(domainName) == False:
    raise "invalid applications dir, domain name must be last folder " + str(applicationsHome)

rcuDbUrl=fdGetEnv('FLX_INSTALL_RCU_DB_URL')
fdLog(thisMethodName, "rcuDbUrl=" + str(rcuDbUrl))

rcuDbUser=fdGetEnv('FLX_INSTALL_RCU_DB_USER')
fdLog(thisMethodName, "rcuDbUser=" + str(rcuDbUser))
if rcuDbUser.upper().endswith('_STB') == False:
    raise "RCU user must end with _STB " + str(rcuDbUser)

rcuDbPassword=fdGetEnv('FLX_INSTALL_RCU_DB_PASSWORD')

# read and update domain with adf related templates
fdReadDomain()

adfTemplateFilePath=deriveADFTemplateFilePath()
fdLog(thisMethodName, "Update domain with - " + str(adfTemplateFilePath))
addTemplate(adfTemplateFilePath)

fdLog(thisMethodName, "Read database services information from RCU schemas")
cd('/JDBCSystemResource/LocalSvcTblDataSource/JdbcResource/LocalSvcTblDataSource/JDBCDriverParams/NO_NAME_0')
set('DriverName', 'oracle.jdbc.OracleDriver')
set('URL', rcuDbUrl)
set('PasswordEncrypted', rcuDbPassword)
cd('Properties/NO_NAME_0/Property/user')
set('Value', rcuDbUser)
 
fdLog(thisMethodName, "Read database services by calling getDatabaseDefaults.")
getDatabaseDefaults()    

fmwEMTemplateFilePath=deriveFMWEMTemplateFilePath()
fdLog(thisMethodName, "Update domain with - " + str(fmwEMTemplateFilePath))
addTemplate(fmwEMTemplateFilePath)

setOption( "AppDir", applicationsHome )

fdUpdateDomain()